from __future__ import annotations

import dataclasses
from typing import Annotated

from pylav.nodes.api.responses.filters.misc import ValueRangeList


@dataclasses.dataclass(repr=True, frozen=True, kw_only=True, slots=True)
class Reverb:
    delays: Annotated[list[float] | None, ValueRangeList(min=0, max=float("inf"))] = None
    gains: Annotated[list[float] | None, ValueRangeList(min=0, max=float("inf"))] = None

    def to_dict(self) -> dict[str, list[float]]:
        response: dict[str, list[float]] = {}
        if self.delays is not None:
            response["delays"] = self.delays
        if self.gains is not None:
            response["gains"] = self.gains
        return response



from __future__ import annotations

import dataclasses
from typing import NotRequired

from pylav.type_hints.dict_typing import JSON_DICT_TYPE


@dataclasses.dataclass(repr=True, frozen=True, kw_only=True, slots=True)
class Info:
    name: str
    selectedTrack: int
    type: NotRequired[str] = "playlist"
    url: NotRequired[str | None] = None
    artworkUrl: NotRequired[str | None] = None
    author: NotRequired[str | None] = None

    def to_dict(self) -> JSON_DICT_TYPE:
        return dataclasses.asdict(self)



from __future__ import annotations

import dataclasses

from pylav.nodes.api.responses.shared import LyricsPluginInfo

__all__ = ("LyricsLine", "LyricsObject")


@dataclasses.dataclass(repr=True, frozen=True, kw_only=True, slots=True)
class LyricsLine:
    timestamp: int
    duration: int | None
    line: str
    plugin: LyricsPluginInfo

    def __post_init__(self):
        if isinstance(self.plugin, dict) and (p := LyricsPluginInfo(**self.plugin)):
            object.__setattr__(self, "plugin", p)


@dataclasses.dataclass(repr=True, frozen=True, kw_only=True, slots=True)
class LyricsObject:
    sourceName: str
    provider: str
    text: str | None = None
    lines: list[LyricsLine]
    plugin: LyricsPluginInfo

    def __post_init__(self):
        if isinstance(self.plugin, dict) and (p := LyricsPluginInfo(**self.plugin)):
            object.__setattr__(self, "plugin", p)



from pathlib import Path

import discord
from redbot.core.i18n import Translator
from redbot.core.utils.chat_formatting import humanize_number
from redbot.vendored.discord.ext import menus

from pylav.extension.red.ui.menus.generic import BaseMenu
from pylav.logging import getLogger
from pylav.players.tracks.obj import Track
from pylav.type_hints.bot import DISCORD_COG_TYPE

_ = Translator("PyLav", Path(__file__))

LOGGER = getLogger("PyLav.ext.red.ui.sources.lyrics")


class LyricsSource(menus.ListPageSource):
    def __init__(self, cog: DISCORD_COG_TYPE, track: Track, pages: list[str]):
        super().__init__(pages, per_page=1)
        self.track = track
        self.cog = cog

    def get_starting_index_and_page_number(self, menu: BaseMenu) -> tuple[int, int]:
        page_num = menu.current_page
        start = page_num * self.per_page
        return start, page_num

    async def format_page(self, menu: BaseMenu, page: str) -> discord.Embed:
        __, page_num = self.get_starting_index_and_page_number(menu)
        total_number_of_entries = len(self.entries)
        current_page = humanize_number(page_num + 1)
        total_number_of_pages = humanize_number(self.get_max_pages())
        title = f"{await self.track.title()}"
        page = await self.cog.pylav.construct_embed(
            messageable=menu.ctx,
            title=title,
            description=page,
            thumbnail=await self.track.artworkUrl(),
            url=await self.track.uri(),
        )
        match total_number_of_entries:
            case 1:
                message = _("Page 1 / 1 | 1 Page")
            case 0:
                message = _("Page 1 / 1 | 0 Pages")
            case __:
                message = _(
                    "Page {current_page_variable_do_not_translate} / {total_number_of_pages_variable_do_not_translate} "
                    "| {total_number_of_entries_variable_do_not_translate} Pages"
                ).format(
                    current_page_variable_do_not_translate=current_page,
                    total_number_of_pages_variable_do_not_translate=total_number_of_pages,
                    total_number_of_entries_variable_do_not_translate=humanize_number(total_number_of_entries),
                )
        page.set_footer(text=message)

        return page



from __future__ import annotations

from typing import TYPE_CHECKING

from deepdiff import DeepDiff

from pylav.constants.node import NODE_DEFAULT_SETTINGS
from pylav.storage.migrations.logging import LOGGER

if TYPE_CHECKING:
    from pylav.core.client import Client


async def fix_managed_node_settings(client: Client) -> None:
    """Fix the managed node settings."""
    LOGGER.info("Running migration - Fixing Managed Node Settings")
    # noinspection PyProtectedMember
    config = client._node_config_manager.bundled_node_config()
    data = await config.fetch_yaml()
    if (ddiff := DeepDiff(data, NODE_DEFAULT_SETTINGS, ignore_order=True, max_passes=3, cache_size=10000)) and (
        ddiff.get("dictionary_item_added") or ddiff.get("dictionary_item_removed")
    ):
        LOGGER.warning("Managed node settings are not IN the expected format - Force resetting it")
        await config.update_yaml(NODE_DEFAULT_SETTINGS)



from __future__ import annotations

from asyncpg import Connection
from packaging.version import parse

from pylav.constants.config import DEFAULT_PLAYER_VOLUME
from pylav.constants.versions import VERSION_1_15_7
from pylav.storage.migrations.logging import LOGGER


async def low_level_v_1_15_7_migration(con: Connection) -> None:
    """Run the low level migration for PyLav 1.10.6."""
    await low_level_v_1_15_7_queries(con)


async def low_level_v_1_15_7_queries(con: Connection) -> None:
    """Run the queries migration for PyLav 1.10.6."""
    await run_queries_migration_v_1_15_7(con)


async def run_queries_migration_v_1_15_7(con: Connection) -> None:
    """
    Add the info and pluginInfo columns to the query table.
    """
    has_column = """
        SELECT EXISTS (SELECT 1
        FROM information_schema.columns
        WHERE table_name='version' AND column_name='version')
        """
    has_version_column = await con.fetchval(has_column)
    if not has_version_column:
        return

    version = await con.fetchval("SELECT version from version;")
    if version is None:
        return

    version_parsed = parse(version)
    if version_parsed >= VERSION_1_15_7:
        return

    migrated = False
    has_column = """
        SELECT EXISTS (SELECT 1
        FROM information_schema.columns
        WHERE table_name='player_state' AND column_name='volume')
        """
    has_volume_column = await con.fetchval(has_column)
    if has_volume_column:
        alter_table = f"""
        ALTER TABLE player_state ALTER COLUMN volume SET DEFAULT {DEFAULT_PLAYER_VOLUME};
                """
        await con.execute(alter_table)
        migrated = True

    has_column = """
            SELECT EXISTS (SELECT 1
            FROM information_schema.columns
            WHERE table_name='player' AND column_name='volume')
            """
    has_column_response = await con.fetchval(has_column)
    if has_column_response:
        alter_table = f"""
        ALTER TABLE player ALTER COLUMN volume SET DEFAULT {DEFAULT_PLAYER_VOLUME};
        """
        await con.execute(alter_table)
        migrated = True

    if migrated:
        LOGGER.info("----------- Migrating table player to PyLav 1.15.7 ---------")
